#ifndef _SEGLOBAL_H_
#define _SEGLOBAL_H_

#pragma warning (disable: 4786)

#include <iostream>
#include <fstream>
#include <string>
#include <vector>
#include <map>
#include <set>

typedef unsigned int UINT;

typedef std::string tTerm;
typedef double tScore;
typedef UINT tDocID;

#ifdef _WIN32
#include <crtdbg.h>
#define ASSERT _ASSERTE
#else /* UNIX */
#include <assert.h>
#define ASSERT assert
#endif


template<class T>
class Vector : public std::vector<T>
{
public:
	Vector() {}

	Vector(size_type size) : std::vector<T>(size) {}

#ifdef _DEBUG
	reference operator[](size_type pos) {
		ASSERT(pos < size());
		return ((std::vector<T>*)this)->operator[](pos);
	}

    const_reference operator[](size_type pos) const {
		ASSERT(pos < size());
		return ((std::vector<T>*)this)->operator[](pos);
	}
#endif _DEBUG    
};

template<class T>
std::ostream& operator<<(std::ostream& s, const Vector<T>& o) {
	s << "[ ";
	int i;
	if (o.size() > 0) {
		for (i = 0; i < o.size() - 1; i++) {
			s << o[i] << " , ";
		}
		s << o[i];
	}
	s << "]";
	return s;
}

template <class Key, class T>
class Map: public std::map<Key, T> {
public:
	bool hasKey(const Key& key) const {
		return (find(key) != end());
	}

	const T& get(const Key& key) const {
		ASSERT(hasKey(key));
		return (const_cast<Map<Key,T>*>(this))->operator[](key);
	}
};


template <class Key, class T>
std::ostream& operator<<(std::ostream& s, const Map<Key, T>& o) {
	s << "{ ";
	for (Map<Key, T>::const_iterator i = o.begin(); i != o.end(); i++) {
		s << i->first << " => " << i->second;
		s << " , ";
	}
	s << "}";
	return s;
}

template <class T>
class Set: public std::set<T> {
public:
	bool hasElement(const T& element) const {
		return (find(element) != end());
	}
};

template <class T>
std::ostream& operator<<(std::ostream& s, const Set<T>& o) {
	s << "{ ";
	for (Set<T>::const_iterator i = o.begin(); i != o.end(); i++) {
		s << *i << ", ";
	}
	s << "}";
	return s;
}

class SEException {
	std::string m_msg;
public:
	SEException(const std::string& msg) : m_msg(msg) {}
	friend std::ostream& operator<<(std::ostream& s, const SEException& e);
};

inline std::ostream& operator<<(std::ostream& s, const SEException& e)
{
	s << "seException: " << e.m_msg;
	return s;
}

#include <stdio.h>
inline std::string operator+(const std::string& s, int i) {
	char st[128];
	sprintf(st, "%d", i);
	return (s+st);
}

#define THROW(msg) throw SEException(std::string(__FILE__) + ":" + __LINE__ + " " + msg)

class RemarkEater {
};

static RemarkEater remarkEater;

inline std::istream& operator>>(std::istream& s, const RemarkEater& remarkEater)
{
	s >> std::ws;
	while (s.peek() == '#') {
	  int ch;
	  while ( ((ch = s.get()) != '\n') && (ch != EOF));
	  s >> std::ws;
	}
	return s;
}

#endif /* _SEGLOBAL_H_ */
