#ifndef _RERANKER_H_
#define _RERANKER_H_

#include <Common/SEGlobal.h>
#include <Matchers/Matcher.h>

/**
 * Structure that represents a reranking result
 */
class tRerankResult : public tMatchResult {
public:
	tScore rerankScore;
	
	tRerankResult() {}

	tRerankResult(const tMatchResult& matchResult, tScore _rerankScore) 
		: tMatchResult(matchResult), rerankScore(_rerankScore) {}

	bool operator<(const tRerankResult& other) const {
		if (matchScore > other.matchScore)
			return true;
		if (matchScore < other.matchScore)
			return false;
		return (rerankScore > other.rerankScore);
	}
};
inline std::ostream& operator<<(std::ostream& s, const tRerankResult& rerankResult) {
	s << "(" << rerankResult.docID << "#" << rerankResult.matchScore << "#" << rerankResult.rerankScore << ")";
	return s;
}

/**
 * Base-class for document rerankers
 *
 * Rerankers are responsible for re-ordering the matched
 * documents s.t. best matching documents are first
 */
class Reranker {
public:
	/**
	 * Reranks the matched documents
	 * @param queryTokens  - Query tokens by which to rerank
	 * @param matchResult  - Result vector from a Matcher
	 * @param rerankResult - Vector for reranked documents
	 *                       (first document is the best)
	 */
	virtual void rerank(const Vector<tTerm>& queryTerms, const Set<tMatchResult>& matchResults,
						Vector<tRerankResult>& rerarkResults) = 0;
};

#endif /* _RERANKER_H_ */