#pragma warning( disable : 4786)

#include <algorithm>

#include "TFIDFReranker.h"
#include <Engine/SearchEngine.h>
#include <math.h>

void TFIDFReranker::rerank(const Vector<tTerm>& queryTerms, const Set<tMatchResult>& matchResults,
						   Vector<tRerankResult>& rerankResults)
{
	rerankResults.clear();

	int i;

	// Generate the query vector (this is actually a map that holds only non-zero terms):
	// Calculate word frequency in the query
	tSpaceVector q; // the query vector
	for (i = 0; i < queryTerms.size(); i++) {
		if (q.hasKey(queryTerms[i]))
			q[queryTerms[i]]++;
		else
			q[queryTerms[i]]=1;
	}
	// Step 2 - Calculate TF.IDF weight for the query
	UINT N = SearchEngine::instance().numIndexedDocs();
	q.vLength = 0;
	for (tSpaceVector::iterator iTerm = q.begin(); iTerm != q.end(); iTerm++) {
		double t_fi = iTerm->second;
		double d_fi = SearchEngine::instance().getNumTermDocuments(iTerm->first);
		double lf_idf;
		if (d_fi > 0)
			lf_idf = iTerm->second = (1+log(t_fi))*log(N/d_fi);
		else
			lf_idf = 0;

		q.vLength += lf_idf * lf_idf;
	}
	q.vLength = sqrt(q.vLength); // q.vLength now holds |q| as supposed to
	
	// Update that score in the queryResultVector according to the cosine between
	// q and each document vector
	tSpaceVector d;
	for (Set<tMatchResult>::const_iterator iMR = matchResults.begin(); iMR != matchResults.end(); iMR++) {
		const tMatchResult& matchResult = *iMR;
		SearchEngine::instance().getDocumentVector(matchResult.docID, d);
		rerankResults.push_back(tRerankResult(matchResult, q.cosine(d)));
	}

	// Sort the queryResultVector according to the combined score of hitScore and cosineScore.
	std::sort(rerankResults.begin(), rerankResults.end());
}
